@@ -597,7 +597,7 @@ class Sale(Process):
             fit[p] = d['fit']
             not_fit[p] = d['not_fit']
             decay[p] = d['decay']
-            csl[p] = d['fit'] / d['all']
+            csl[p] = d['fit'] / d['all'] if d['all'] != 0 else 0
             inventory[p] = store.prod_total[p] if p in store.prod_total else 0
             sales_volume[p] = d['sales_volume']
             backlog_penalty[p] = d['backlog_penalty']
@@ -614,7 +614,7 @@ class Sale(Process):
                           "inventory values": inventory,
                           "unit price s": unit_price,
                           "sales volume s": sales_volume,
-                          "csl s": csl, "current time":self.process_unit.get_time()})
+                          "csl s": csl})
 
     @staticmethod
     def load(json, process_unit):
@@ -682,7 +682,8 @@ class Scheduling(Process):
         super().reset()
         self.obs = []
         self.idle_cnt = 0
-        for i in range(self.prod_feature_dim * len(self.scheduling_valid_prod) + self.env_feature_dim):
+        available_actions = self.process_unit.env.get_available_actions()
+        for i in range(self.prod_feature_dim * len(available_actions) + self.env_feature_dim):
             self.obs.append(0)
         return self.obs
 
@@ -695,10 +696,10 @@ class Scheduling(Process):
         self.env_obs_define = obs_define["env"]
         self.env_feature_dim = len(self.env_obs_define)
         self.reward_define = reward_define
-        self.scheduling_valid_prod = []
+        available_actions = []
         for sub_id in Substance.substance:
-            if sub_id.get_level_value(1) not in self.scheduling_valid_prod:
-                self.scheduling_valid_prod.append(sub_id.get_level_value(1))
+            if sub_id.get_level_value(1) not in available_actions:
+                available_actions.append(sub_id.get_level_value(1))
 
         self.last_scheduling_time = 0
         self.last_profit = 0
@@ -708,7 +709,7 @@ class Scheduling(Process):
         self.action = None
         self.obs = []
         self.idle_cnt = 0
-        for i in range(self.prod_feature_dim * len(self.scheduling_valid_prod) + self.env_feature_dim):
+        for i in range(self.prod_feature_dim * len(available_actions) + self.env_feature_dim):
             self.obs.append(0)
 
     def on_new_scheduling(self, action):
@@ -734,7 +735,8 @@ class Scheduling(Process):
         #                                   "inventory values": inventory,"delta profit": self.delta_profit,
         #                                   "csl s": csl, "last_action":last_action,"action":action,
         #                                   "delta time":delta_time, "current time":self.process_unit.get_time()})
-        for p in self.scheduling_valid_prod:
+        available_actions = self.process_unit.env.get_available_actions()
+        for p in available_actions:
             self.set_obs_inf(self.obs_inf_k, {"p": p}, replace=False)
             for product_obs_var in self.product_obs_define:
                 self.obs.append(self.process_unit.env.variables.get_variable(product_obs_var))
@@ -742,6 +744,10 @@ class Scheduling(Process):
             self.obs.append(self.process_unit.env.variables.get_variable(env_obs_ver))
 
     def scheduling(self, action):
+        try:
+            action = int(action)
+        except TypeError:
+            pass
         if isinstance(action, int):
             if self.process_unit.needs_scheduling():
                 self.on_new_scheduling(action)
@@ -750,9 +756,14 @@ class Scheduling(Process):
             self.refresh_obs_inf_keys()
             self.process_unit.get_latter_unit().process(action)
             self.process_unit.env.time += 1
+            if True:
+                self.set_obs_inf("available_actions", {self.process_unit.env.time:self.process_unit.env.get_available_actions()},
+                                 replace=False)
+                self.set_obs_inf("selected_action", {self.process_unit.env.time:action},
+                                 replace=False)
             if action == 0:
                 self.idle_cnt += 1
-            self.set_obs_inf("scheduling", {"time": self.process_unit.env.time})
+            self.set_obs_inf(self.obs_inf_k, {"current time":self.process_unit.get_time()}, replace=False)
             self.set_obs_inf(self.obs_inf_k, {'idle percent': self.idle_cnt / self.process_unit.env.time}, replace=False)
             done = self.process_unit.get_time() >= self.get_distribution("scheduling_period", None).sample()
             if self.process_unit.needs_scheduling() or done:
