@@ -14,8 +14,12 @@ class ProcessUnit(ABC):
         self.env = env
         self.process_start_time = env.time
         self.last_process_start_time = env.time
+        self.last_process_end_time = 0
         self.m_id = m_id
         self.model_json = model_json
+    
+    def __setattr__(self, key, value):
+        self.__dict__[key] = value
 
     def get_time(self):
         return self.env.time - self.process_start_time
@@ -47,12 +51,24 @@ class ProductionUnit(ProcessUnit):
     product_units = {}
 
     def reset(self):
+        lead_process_finish_time = -1
         if self.setup is not None:
+            lead_process_finish_time = self.setup.finish_time
             self.setup.reset()
         if self.launch is not None:
-            self.launch.reset()
+            not_finish = self.launch.finish_time == -1 or self.launch.finish_time == 0
+            if not_finish == -1 and lead_process_finish_time != -1:
+                self.launch.reset(True, lead_process_finish_time)
+                lead_process_finish_time = self.get_time()
+            else:
+                lead_process_finish_time = self.launch.finish_time
         if self.conversion is not None:
-            self.conversion.reset()
+            not_finish = self.conversion.finish_time == -1 or self.conversion.finish_time == 0
+            if not_finish and lead_process_finish_time != -1:
+                self.conversion.reset(True, lead_process_finish_time)
+                lead_process_finish_time = self.get_time()
+            else:
+                lead_process_finish_time = self.conversion.finish_time
         if self.failure is not None:
             self.failure.reset()
         self.idle = True
@@ -74,6 +90,7 @@ class ProductionUnit(ProcessUnit):
             self.conversion.set_obs_inf(self.conversion.obs_inf_k, {'idle': value}, False)
 
     def process(self, material: Substance):
+        # print(material.m_id.get_level_value(1))
         if material.model_json is None:
             self.idle = True
             if self.get_latter_unit() is not None:
@@ -95,8 +112,11 @@ class ProductionUnit(ProcessUnit):
             if self.failure is not None:
                 failed = self.failure.process(material, self.launch)
                 if failed:
+                    # if self.get_time() < 20:
+                    #     print('failed', self.get_time())
                     self.reset()
                     self.get_latter_unit().reset()
+                    self.last_process_end_time = self.env.time
                     return failed
 
             finished = self.conversion.process(material, self.launch) if self.conversion is not None else True
@@ -109,10 +129,11 @@ class ProductionUnit(ProcessUnit):
                 if self.setup is not None:
                     self.setup.reset()
                 if self.launch is not None:
-                    self.launch.reset()
+                    self.launch.reset(False)
                 if self.conversion is not None:
-                    self.conversion.reset()
+                    self.conversion.reset(False)
                 self.idle = True
+                self.last_process_end_time = self.env.time
             return finished
 
     @staticmethod
@@ -177,6 +198,7 @@ class BatchProductionUnit(ProductionUnit):
             if c_p_u.idle:
                 return c_p_u
         c_p_u = ContinuousProductionUnit(self.m_id, self.model_json, self.env)
+        setattr(c_p_u, 'batch', True)
         self.c_p_u_list.append(c_p_u)
         return c_p_u
 
